# THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY.

from typing import Any, Callable, Tuple

from ruamel.yaml import comments
from .base import Calico

SpecNode = comments.CommentedMap

def get_comment_value(node: SpecNode, name: str, field: str) -> str: ...
def get_attribute(
    node: SpecNode,
    test_name: str,
    names: Tuple[str],
    val_func: Callable[[Any, ...], bool],
    val_args: Any,
    err_message: str,
) -> Any: ...
def parse_spec(content: str) -> Calico: ...
