# THIS FILE IS AUTOMATICALLY GENERATED, DO NOT EDIT MANUALLY.

from typing import List, Optional

from argparse import ArgumentParser

def make_parser(prog: str) -> ArgumentParser: ...
def setup_logging(debug: bool, log: bool) -> None: ...
def main(argv: Optional[List[str]] = ...) -> None: ...
